;************************************************************************************
;* Programacion de Generador de Caracteres Binarios                                 *
;* Para prueba de Multiplexor de 8 canales                                          *
;* Autor : Hugo Gutierrez Salazar                                                   *
;* Fecha : Febrero 2005                                                             *
;* Procesador : PIC16F84A/04P                                                       *
;************************************************************************************
;* Puerto A : (entradas)                                                            *
;*      Botones de Input                                                            *
;*                                                                                  *
;*   A0    :  > Incrementa tiempo de barrido                                        *
;*   A1    :  Genera tren de pulsos en linea                                        *
;*   A2    :  Genera tren de pulsos en puntos                                       *
;*   A3    :  Genera cuenta Hexadecimal                                             *
;*   A4    : < Decrementa tiempo de barrido                                         *
;*                                                                                  *
;************************************************************************************
;*   Reset : Resetea Procesador (Sin programacion)                                  *
;************************************************************************************
;* Puerto B : (Salidas)                                                             *
;*       Leds de Cuenta                                                             *
;*   B0    :  Cuenta Binaria menos significativa 2**0 = 1                           *
;*   B1    :  Cuenta Binaria                     2**1 = 2                           *
;*   B2    :  Cuenta Binaria                     2**2 = 4                           *
;*   B3    :  Cuenta Binaria                     2**3 = 8                           *
;*   B4    :  Cuenta Binaria menos significativa 2**4 = 16                          *
;*   B5    :  Cuenta Binaria menos significativa 2**5 = 32                          *
;*   B6    :  Cuenta Binaria menos significativa 2**6 = 64                          *
;*   B7    :  Cuenta Binaria mas significativa   2**7 = 128                         *
;*                                                                                  *
;************************************************************************************
       LIST p=16F84a;
       #include "p16f84a.inc"
       __config 0x3FF9 ; XT , Power Up off ; Watchdog off ; Codeprotect off
;
;            Definicion de Constantes
;
w         equ  0    ;
on        equ  0    ; Estado activo de un boton
off       equ  1    ; Estado inactivo de un boton
cuarenta  equ  .40  ; Constante decimal 40
cero      equ  0    ; Constante cero
uno       equ  1    ; Constante uno
dos       equ  2    ; Constante dos
tres      equ  3    ; Constante tres
cinco     equ  5    ; Constante cinco
diez      equ  10h  ; Constante diez hexadecimal
PORTA     equ  05h  ; 05h Banco 0
PORTB     equ  06h  ; 06h Banco 0
STATUS    equ  03h  ; Registro de Estados
RP0       equ  5    ;  Bit registro de Estados Selecciona Banco de Memoria
ff        equ  .255 ; constante ff
;
valor1    equ  7     ; Valores establecidos 
valor2    equ  8     ; para realizar retardos de tiempos
valor3    equ  9     ; del conteo.
;            la letra "O" indica el boton presionado (visto desde afuera).
;
boton1 equ  0      ;    X  X  X  X  O     Incrementa tiempo de Barrido (mas lento)
boton2 equ  1      ;    X  X  X  O  X     Genera pulsos en linea
boton3 equ  2      ;    X  X  O  X  X     Genera pulsos en puntos
boton4 equ  3      ;    X  O  X  X  X     Genera Cuenta Hexadecimal
boton5 equ  4      ;    O  X  X  X  X     Decrementa tiempo de Barrido (mas rapido)
;
;            Posiciones de Memoria
;
CuentaHex   equ   0x1C  ; Variable Cuenta Hexadecimal
CuentaDec   equ   0x1D  ; Variable Cuenta Decimal (Sin uso por el momento)
linea       equ   0x1E  ; byte de despliegue completo
;
reg1        equ   0x1F  ; Variable para guardar cuenta de retardo
reg2        equ   0x20  ; Variable para guardar cuenta de retardo
reg3        equ   0x21  ; Variable para guardar cuenta de retardo
;
dectiempo   equ   0x22  ; Variable paso decremento tiempo en delay

funcion     equ   0x23  ; Tipo de cuenta que realizara

; Si funcion = 1 : Linea Continua
; Si funcion = 2 : Linea de Puntos
; Si funcion = 3 : Cuenta Binaria Hexadecimal

tiempo      equ   0x24  ; Variable que contiene tiempo de retardo

;***********************************************************************
;            Programa Principal
;***********************************************************************
;
;   Configura el puerto A como entradas
;
        clrf PORTA      ; Limpia la memoria de datos
        movlw  b'11111' ; Mueve 11111 a w indicando que seran pines de entradas
        bsf STATUS, RP0 ; Selecciona el Banco de Memoria 1
        movwf TRISA     ; Setea el puerto A.
;
;   Configura el Puerto B como Salidas
;
        bcf STATUS, RP0    ; Vuelve al Banco 0
        clrf PORTB         ; Limpia memoria de datos de salida
        bsf STATUS, RP0    ; Selecciona el banco 1
        movlw 0x00         ; Mueve ceros a w
        movwf TRISB        ; Resetea el puerto B
        bcf STATUS, RP0    ; Vuelve al banco 0


inicio  movlw 0x02         ; w <-2 
        movwf funcion      ; Setea la funcion a 2
        movlw 0x01         ; w <- 1
        movwf linea        ; Setea linea en 1
        movwf CuentaHex    ; Setea CuentaHex en 1

        movlw 0x10         ; w <- 10
        movwf tiempo       ; setea el valor de tiempo inicial en 10 Hex
vuelve
        btfsc PORTA,boton1 ; Si se ha presionado el boton salta proxima instruccion
        goto  b2           ; Va a Testear el boton 2
;
;       Rutina del Boton 1 : Incrementa el tiempo de barrido (mas lento)
;
        movf tiempo,w      ; w <- tiempo
        sublw diez         ; verifica si la cuenta llego a 10 (w <- 10 - tiempo)
        bnz   sigue1       ; si no es 10
        goto continua      ; Sigue programa

sigue1  incf tiempo,w      ; incrementa el tiempo en 1 ( w <- tiempo +1)
        movwf tiempo       ; tiempo <- w
        goto continua      ; Sigue programa

b2      btfsc PORTA,boton2  ; Si se ha presionado el boton 2, salta proxima instruccion
        goto   b3           ; Va a testear el boton 3
;
;       Rutina del Boton 2  : Linea Continua
;
        movlw uno           ; w <- 1
        movwf funcion       ; Setea la funcion a 1
        movlw 0x01          ;  w <- 1
        movwf linea         ; Setea linea en 1

        goto continua       ; sigue programa
b3      btfsc PORTA, boton3 ; Si se ha presionado el boton 3 salta proxima instruccion
        goto b4             ; Va a testear el boton 4
;
;       Rutina del Boton 3  : Linea de puntos
;
        movlw  dos          ; w <- 2
        movwf funcion       ; Setea la funcion a 2
        movlw 0x01          ;  w <- 1
        movwf linea         ; Setea linea en 1
        goto  continua      ; Continua Programa

b4      btfsc PORTA, boton4 ; Si se ha presionado el boton 4 salta proxima instruccion
        goto b5             ; Va a testear el boton 5
;
;        Rutina del boton 4  : Cuenta Hexadecimal
;

         movlw  tres        ; w <- 3
         movwf funcion      ; Setea la funcion a 3
         goto continua      ; Sigue programa

b5       btfsc PORTA, boton5 ; Si se ha presionado el boton 5 salta proxima instruccion
         goto continua       ; Se acabaron botones, vuelve a iniciar el ciclo de test
;
;        Rutina del boton 5 : Decrementa el tiempo de barrido (mas rapido)
;
        movf tiempo,w      ; w <- tiempo
        sublw cero         ; verifica si la cuenta llego a 0 (w <- 0 - tiempo)
        bnz   sigue2       ; si no es 0
        goto continua      ; sigue programa

sigue2  decf tiempo,w      ; decrementa el tiempo en 1 ( w <- tiempo -1)
        movwf tiempo       ; tiempo <- w
        goto continua      ; Sigue programa.
;
;   sigue programa normalmente
;
continua
; Si la funcion es 1 : Linea Continua
        movf funcion,w     ; w <- funcion
        sublw uno          ; verifica si es funcion 1
        bnz fun2           ; si no es sigue en funcion2
        
; Si es funcion 1
        call display1      ; muestra el contenido de la variable

; Si la cuenta es 1111 1111 (FF)

        movf linea,w       ; mueve linea a w
        sublw 0xff         ; w < 0xff - linea
        bnz sig8           ; si no es ff, sigue preguntando
        movlw 0x7f         ; Si la cuenta era ff, le muevo 7f
        movwf linea        ; linea <- 7f
        goto vuelve        ; reinicia el proceso

; si la cuenta es 0111 1111 (7F)

sig8    movf linea,w       ; mueve linea a w
        sublw 0x7f         ; w < 0x7f - linea
        bnz sig7           ; Si no es 7f , sigue preguntando
        movlw 0x3f         ; Si la cuenta era 7f , le muevo 3f
        movwf linea        ; linea <- 3f
        goto vuelve        ; reinicia proceso

; Si la cuenta es 0011 1111 (3F)

sig7    movf linea,w       ; mueve linea a w
        sublw 0x3f         ; w < 0x3f - linea
        bnz sig6           ; si no es 3f, sigue preguntando
        movlw 0x1f         ; Si la cuenta era 3f , le muevo 1f
        movwf linea        ; linea <- 1f
        goto vuelve        ; reinicia proceso

; Si la cuenta es 0001 1111 (1F)

sig6    movf linea,w       ; mueve linea a w
        sublw 0x1F         ; si w < 0x1f - linea
        bnz sig5           ; si no es 1f, sigue preguntando
        movlw 0x0f         ; Si la cuenta era 1f, le muevo 0f
        movwf linea        ; linea <- 0f
        goto vuelve        ; reinicia el proceso

;Si la cuenta es 0000 1111 (0F)

sig5    movf linea,w       ; mueve linea a w 
        sublw 0x0f         ; si w < 0x0f - linea
        bnz sig4           ; Si no es 0f, sigue preguntando
        movlw 0x07         ; si la cuenta era 0f , le muevo 07
        movwf linea        ; linea <- 07
        goto vuelve        ; reinicia el proceso

;Si la cuenta es 0000 0111 (07)

sig4    movf linea,w       ; mueve linea a w
        sublw 0x07         ; si w < 0x07 - linea
        bnz sig3           ; Si no es 07, sigue preguntando
        movlw 0x03         ; si la cuenta era 07 , le muevo 03
        movwf linea        ; linea <- 04
        goto vuelve        ; reinicia el proceso          
    
;Si la cuenta es 0000 0011 (03)

sig3    movf linea,w       ; mueve linea a w
        sublw 0x03         ; si w < 0x03 - linea
        bnz sig2           ; Si no es 03, sigue preguntando
        movlw 0x01         ; si la cuenta era 03 , le muevo 01
        movwf linea        ; linea <- 01
        goto vuelve        ; reinicia el proceso

;Si la cuenta es 0000 0001 (01)

sig2    movf linea,w       ; mueve linea a w
        sublw 0x01         ; si w < 0x01 - linea
        bnz sig1           ; Si no es 01, vuelve
        movlw 0x00         ; si la cuenta era 01 , le muevo 0
        movwf linea        ; linea <- 00
        goto vuelve        ; reinicia el proceso

;Si la cuenta es 0000 0000 (00)

sig1    movf linea,w       ; mueve linea a w
        sublw 0x00         ; Si w < 0x00 - linea
        bnz vuelve         ; si no es 0 , vuelve
        movlw 0xFF         ; Si la cuenta es 00, le muevo FF
        movwf linea        ; linea <- FF
        goto vuelve

; Si la funcion es 2 : Linea de Puntos

fun2    movf funcion, w    ; w <- funcion
        sublw dos          ; verifica si es funcion 2
        bnz fun3           ; si no es sigue en funcion3
        call display1      ; muestra el contenido de la variable

; Si linea es 00h   (0000 0000)

        movf linea,w       ; mueve linea a w
        sublw 0x00         ; si w < 0x00 - linea
        bnz p1             ; Si no es 00, pregunta si es 1
        movlw 0x01         ; si la cuenta era 00 , le mueve 01
        movwf linea        ; linea <- 01
        goto vuelve        ; reinicia el proceso

; Si la linea es 01h (0000 0001)
        
p1      movf linea,w       ; mueve linea a w
        sublw 0x01         ; si w es 01
        bnz p2             ; si no es 01 pregunta si es 10
        movlw 0x02         ; si la cuenta era 01 le mueve 2h
        movwf linea        ; linea <- 02
        goto vuelve        ; reinicia proceso

; Si la linea es 02h (0000 0010)

p2      movf linea,w       ; mueve linea a w
        sublw 0x02         ; si w es 02h
        bnz p3             ; si no es 02 pregunta si es 04h
        movlw 0x04         ; si la cuenta era 02 le mueve 4h
        movwf linea        ; linea <- 04h
        goto vuelve        ; reinicia proceso

; Si la linea es 04h (0000 0100)

p3      movf linea,w       ; mueve linea a w
        sublw 0x04         ; si w es 04h
        bnz p4             ; si no es 04h pregunta si es 08h
        movlw 0x08         ; si la cuenta era 04h le mueve 08h
        movwf linea        ; linea <- 08h
        goto vuelve        ; reinicia proceso

; Si la linea es 08h (0000 1000)

p4      movf linea,w       ; mueve linea a w
        sublw 0x08         ; si w es 08h
        bnz p5             ; si no es 08h pregunta si es 10h
        movlw 0x10         ; si la cuenta era 08h le mueve 10h
        movwf linea        ; linea <- 10h
        goto vuelve        ; reinicia proceso

; Si la linea es 10h (0001 0000)

p5      movf linea,w       ; mueve linea a w
        sublw 0x10         ; si w es 10h
        bnz p6             ; si no es 10h pregunta si es 20h
        movlw 0x20         ; si la cuenta era 10h le mueve 20h
        movwf linea        ; linea <- 20h
        goto vuelve        ; reinicia proceso

; Si la linea es 20h (0010 0000)

p6      movf linea,w       ; mueve linea a w
        sublw 0x20         ; si w es 20h
        bnz p7             ; si no es 20h pregunta si es 40h
        movlw 0x40         ; si la cuenta era 20h le mueve 40h
        movwf linea        ; linea <- 40h
        goto vuelve        ; reinicia proceso

; Si la linea es 40 h (0100 0000)

p7      movf linea,w       ; mueve linea a w
        sublw 0x40         ; si w es 40h
        bnz p8             ; Si no es 40h pregunta si es 80h
        movlw 0x80         ; si la cuenta era 40h le mueve 80h
        movwf linea        ; linea <- 80h
        goto vuelve        ; reinicia proceso

; Si la linea es 80 h (1000 0000)

p8      movf linea,w       ; mueve linea a w
        sublw 0x80         ; si w es 80h
        bnz vuelve         ; reinicia proceso
        movlw 0x00         ; si la cuenta era 80h le mueve 00h
        movwf linea        ; linea <- 00h
        goto vuelve        ; reinicia proceso

; Si la funcion es 3 : Cuenta Hexadecimal

fun3    
        call display2       ; Muestra el contenido de la variable

        incf CuentaHex,w    ; Incrementa en 1 la cuenta
        movwf CuentaHex     ; CuentaHex <- w
        goto vuelve         ; reinicia proceso

;**************************************************************************************
;                                   SUBRUTINAS
;**************************************************************************************
;        Subrutina de Retardo de tiempo
;

delay     movf  tiempo,w      ; w <- tiempo
          movwf dectiempo     ; dectiempo <- w
delay1    call retardo        ; retardo de tiempo

;          movf  PORTA,w       ; mueve PORTA a w
;          sublw b'11111'      ; si w es '11111' sigue con retardo de tiempo
;          bnz sale            ; sale de rutina

          decfsz dectiempo,1  ; resta 1 a dectiempo, si la cuenta es != 0
          goto delay1          ; vuelve a realizar el retardo
          return              ;
;

retardo   
          movlw  valor1       ;  w <-  5
          movwf  reg1         ;  reg1 <-  w

loop3     movlw  valor2       ;  w <-  5
          movwf  reg2         ;  reg2 <-  w

loop2     movlw  valor3       ;  w <- 5
          movwf  reg3         ;  reg3 <-  w

loop1     decfsz reg3,1       ; resta 1 a reg3, si la cuenta es !=0
          goto loop1          ; vuelve a loop1 

          decfsz reg2,1       ; resta 1 a reg2, si la cuenta es != 0
          goto loop2          ; vuelve a loop2

          decfsz reg1,1       ; resta 1 a reg1, si la cuenta es != 0
          goto loop3          ; vuelve a loop3
 
          return              ;
;************************************************************************
;      Subrutinas de Despliegue:
;                 Display 1 : Linea o Puntos
;                 display 2 : Cuenta Hexadecimal
;************************************************************************

display1  movf linea,w        ; mueve linea a w
          movwf PORTB         ; Mueve el contenido de linea a la salida B
          call delay          ; llama a rutina de retardo para el despliegue.
          return

display2  movf CuentaHex,w    ; mueve CuentaHex a w
          movwf PORTB         ; Mueve el contenido de linea a la salida B
          call delay          ; llama a rutina de retardo para el despliegue.
          return
          END                 ; Fin para el compilador
